<?php
/*  Copyright (C) 2012  byMike <m at byMike.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Description of eventsbymike_controller_postMetabox
 *
 * @author byMike <m at byMike.ch>
 * 
 */
class eventsbymike_controller_postMetabox extends wpbymike_controller
{
    
    /**
     * Render Meta Box content
     */
    public function getMetabox($post) {
        $this->renderMetabox($post->ID);
    }

    /**
     * Shows the Post Meta Box Content
     * @param int $postId Post_ID
     * @param array $msgbox array of Messages to Show within the Box
     */

    private function renderMetabox($postId, $msgbox = array()) {

        // MessageBox Content
        if (!is_array($msgbox)) $msgbox = array();
        $this->view()->registerVar('msgbox', $msgbox);

        // Post ID
        $this->view()->registerVar('postId', $postId);



        // get Dates
        $date_model = new eventsbymike_model_date($this->pluginReference);
        $dates = $date_model->getAllByPostId($postId, 'array');

        // Check for Results
        if (is_array($dates)) {
            $this->view()->registerVar('dates', $dates);
        } else {
            $this->view()->registerVar('dates', array());
        }

        // Render View

        $this->view()->render('metaboxes/postDates.php');
    }
    
    // AJAX-CONTROLLERS

    public function ajaxDeletedate() {

        if (!wp_verify_nonce($_POST['eventsbymike_nonce'], 'eventsbymike-metabox')) {
            echo "eventsbymike:: nonce verification failed";
            die();
        }

        if (!current_user_can('edit_post', $_POST['postid'])) {
            echo "eventsbymike:: no permission to delete a date";
            die();
        }

        // Prepare Message Array
        $retMessage = array();

        // Get the date_id
        $date_id = $_POST['dateid'];

        // Create Empty Model
        $row2del = new eventsbymike_model_date($this->pluginReference);

        // Try to delete
        if ($row2del->delete($date_id)) {
            $retMessage[] = "Termin entfernt.";
        } else {
            $retMessage[] = "Termin konnte nicht entfernt werden.";
            $retMessage[] = "Vars: " . $date_id;
            $this->getwpdb()->print_error();
            // TODO Hide Debug Message
        }


        $this->renderMetabox($_POST['postid'], $retMessage);



        die(); // this is required to return a proper result

    }

    public function ajaxSaveDates() {

        // Get Formdata and Unserialize it
        $formdata = array();
        parse_str($_POST['formdata'], $formdata);

        // get Post ID
        $post_id = $formdata['post_ID'];

        //print_r($formdata);


        if (!wp_verify_nonce($formdata['eventsbymike_nonce'], 'eventsbymike-metabox')) {
            echo "eventsbymike:: nonce verification failed";
            die();
        }

        if (!current_user_can('edit_post', $post_id)) {
            echo "eventsbymike:: no permission to save dates";
            die();

        }

        // Prepare Message Array
        $retMessage = array();
        $dates = array();

        $newdates = $formdata['datebyMike'];

        if (count($newdates) > 0) {
            foreach ($newdates as $newdate) {
                if($newdate['date_begin'] <> '0000-00-00') {
                    // create the model
                    $datemodel = new eventsbymike_model_date($this->pluginReference, $newdate);
                    $datemodel->setPostId($post_id);
                    if (!$datemodel->save()) {
                        $retMessage[] = $datemodel->getDateBegin() . ' konnte nicht gespeichert werden';
                    }
                    $dates[] = $datemodel->getArray();
                }
            }
        }

        // MessageBox Content
        $this->view()->registerVar('msgbox', $retMessage);

        // Post ID
        $this->view()->registerVar('postId', $post_id);

        // Check for Results
        if (is_array($dates)) {
            $this->view()->registerVar('dates', $dates);
        } else {
            $this->view()->registerVar('dates', array());
        }

        // Render View

        $this->view()->render('metaboxes/postDates.php');


        die(); // this is required to return a proper result

    }

    public function ajaxCheckdate() {

        // TODO flush Output --> Ajax

        if ( wpbymike_filter_mysql::checkdate($_POST['date']) ) echo "1";
        else echo "0";

        die(); // this is required to return a proper result
    }
    
    public function ajaxAutocomplete() {
        
        $model = new eventsbymike_model_date($this->pluginReference);
        
        $locations = $model->getLocations($_REQUEST['input']);
        
        $i = 0;
        $responseList = array();

        foreach ($locations as $location) {
            $responseList[$i] = array (
                'id' => $i,
                'value' => $location,
                'info' => ''
            );
            $i++;
        }
        
        echo json_encode(array('results' => $responseList));

        die(); // this is required to return a proper result
    }
}

// EoC eventsbymike_controller_postMetabox