<?php

/*  Copyright (C)2012 byMike <m at byMike.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Date Model Class
 * DB <-> Date Models
 *
 * @author byMike <m at byMike.ch>
 * 
 */
class eventsbymike_model_date {
    
    /* @var $date_id int */
    private $date_id;
    /* @var $post_id int */
    private $post_id;
    /* @var $date_begin string */
    private $date_begin;
    /* @var $location_name string */
    private $location_name;
    
    private $date_time_begin;
    
    private $date_duration;

    /* @var $presets array Default Values */
    private $presets = array(
        'date_id' => NULL,
        'post_id' => NULL,
        'date_begin' => '0000-00-00',
        'date_time_begin' => '19:00',
        'date_duration' => '02:00',
        'location_name' => ''
    );

    /* @var $eventsbymike obj PlugIn Controller */
    private $eventsbymike;

    /* @var $wpdb obj WordPress DB Connector */
    private $wpdb;

    /* @var $table_name string Dates Table Name */
    private $table_name;

    /**
     * Constructor
     * @param eventsbymike $eventsbymike
     * @param array $values 
     */
    public function __construct(eventsbymike $eventsbymike, $values = array()) {
        // Load ParentObj
        $this->eventsbymike = $eventsbymike;

        // Load WP DataBase Connector
        $this->wpdb = $eventsbymike->getwpdb();

        // set table name
        $this->table_name = $this->eventsbymike->getOption('db_table_name');

        // Set Values from Array
        $values = $this->setFromArray($values);
    }

    // DB Actions

    /**
     * Get Row by ID from DB
     * @param integer $id
     * @return \eventsbymike_model_date 
     */
    public function getById($id) {

        // get the date
        $date = $this->wpdb->get_row(
                $this->wpdb->prepare(
                        "SELECT      *
                        FROM        {$this->table_name}
                        WHERE       date_id = %d",
                                $id),
                                ARRAY_A
                                );
        
        // set the class
        $this->setFromArray($date);

        // return this for further usage
        return $this;
    }

    /**
     * Get All Rows by Post_ID from DB
     * @param integer $post_id
     * @param string Default: Model | Array
     * @return array An array of Models / an 2d array
     */
    public function getAllByPostId($post_id, $return_type = 'model') {

        // check return type
        switch ($return_type) {
            case 'model':
                $return_type = FALSE;
                break;
            case 'array':
                $return_type = TRUE;
                break;
            default :
                $return_type = FALSE;
        }


        // get the date
        $dates = $this->wpdb->get_results(
                $this->wpdb->prepare("
                                SELECT      *
                                FROM        {$this->table_name}
                                WHERE       post_id = %d
                                ORDER BY date_begin ASC
                        ", $post_id), ARRAY_A);

        // set the classes
        $retval = array();
        if (is_array($dates)) {
            for ($i=0;$i<count($dates);$i++) {
                // create models
                $retval[$i] = new eventsbymike_model_date($this->eventsbymike);
                $retval[$i]->setFromArray($dates[$i]);
                
                if ($return_type) {
                    $retval[$i] = $retval[$i]->getArray();
                }
            }
        } else {
            return FALSE;
        } 

        // array-set of classes/arrays
        return $retval;
    }
    
    /**
     * Get the entire Table
     * @param string $return_type 'model' || 'array'
     * @param integer $limit
     * @param string $order by 'ASC' || 'DESC'
     * @return array array of models/ 2d-array 
     */
    public function getAll($return_type = 'model', $limit = 0, $order = 'ASC') {

        // check return type
        switch ($return_type) {
            case 'model':
                $return_type = FALSE;
                break;
            case 'array':
                $return_type = TRUE;
                break;
            default :
                $return_type = FALSE;
        }
        // Prepare the SQL Query
        $queryString = "SELECT * FROM {$this->table_name}";
        
        
        // INNER JOIN
        $queryString .= " INNER JOIN " . $this->wpdb->posts;
        $queryString .= " ON " . $this->wpdb->posts . ".ID = " . $this->table_name . ".post_id";
        
        // WHERE STATE
        $queryString .= " WHERE "  . $this->wpdb->posts . ".post_status = 'publish'";
        
        // ORDER BY
        $queryString .= " ORDER BY date_begin ";
        if ($order == 'DESC')  $queryString .= 'DESC';
        else $queryString .= 'ASC';
        
        if ($limit > 0)
            $queryString .= " LIMIT 0," . $limit;

        // get the dates
        $dates = $this->wpdb->get_results($queryString, ARRAY_A);

        // set the classes
        $retval = array();
        if (is_array($dates)) {
            for ($i=0;$i<count($dates);$i++) {
                // create models
                $retval[$i] = new eventsbymike_model_date($this->eventsbymike);
                $retval[$i]->setFromArray($dates[$i]);
                
                if ($return_type) {
                    $retval[$i] = $retval[$i]->getArray();
                }
            }
        } else {
            return FALSE;
        } 

        // array-set of classes/arrays
        return $retval;
    }
    
    /**
     * Get Vita
     * @param string $return_type 'model' || 'array'
     * @param integer $limit
     * @param string $order by 'ASC' || 'DESC'
     * @return array array of models/ 2d-array 
     */
    public function getVita($return_type = 'model', $limit = 0, $order = 'DESC') {

        // TODO VITA!!!!
        
        // check return type
        switch ($return_type) {
            case 'model':
                $return_type = FALSE;
                break;
            case 'array':
                $return_type = TRUE;
                break;
            default :
                $return_type = FALSE;
        }
        // Prepare the SQL Query, Aggregate Function
        $queryString = "SELECT *, MIN(`date_begin`) FROM {$this->table_name}";
        
        
        // INNER JOIN
        $queryString .= " INNER JOIN " . $this->wpdb->posts;
        $queryString .= " ON " . $this->wpdb->posts . ".ID = " . $this->table_name . ".post_id";
        
        // WHERE STATE
        $queryString .= " WHERE "  . $this->wpdb->posts . ".post_status = 'publish'";
        
        // GROUP BY
        $queryString .= " GROUP BY `post_id`";
        
        
        // ORDER BY
        $queryString .= " ORDER BY date_begin ";
        if ($order == 'DESC')  $queryString .= 'DESC';
        else $queryString .= 'ASC';
        
        if ($limit > 0)
            $queryString .= " LIMIT 0," . $limit;

        // get the dates
        $dates = $this->wpdb->get_results($queryString, ARRAY_A);

        // set the classes
        $retval = array();
        if (is_array($dates)) {
            for ($i=0;$i<count($dates);$i++) {
                // create models
                $retval[$i] = new eventsbymike_model_date($this->eventsbymike);
                $retval[$i]->setFromArray($dates[$i]);
                
                if ($return_type) {
                    $retval[$i] = $retval[$i]->getArray();
                }
            }
        } else {
            return FALSE;
        } 

        // array-set of classes/arrays
        return $retval;
    }
    
    /**
     * Get the future Dates
     * @param string $return_type 'model' || 'array'
     * @param integer $limit
     * @return array array of models/ 2d-array 
     */
    public function getAllInFuture($return_type = 'model', $limit = 0) {

        // check return type
        switch ($return_type) {
            case 'model':
                $return_type = FALSE;
                break;
            case 'array':
                $return_type = TRUE;
                break;
            default :
                $return_type = FALSE;
        }
        
        // Prepare the SQL Query
        $queryString = "SELECT * FROM {$this->table_name}";
        
        // INNER JOIN
        $queryString .= " INNER JOIN " . $this->wpdb->posts;
        $queryString .= " ON " . $this->wpdb->posts . ".ID = " . $this->table_name . ".post_id";
        
        // WHERE STATE
        $queryString .= " WHERE `date_begin` > CURDATE()";
        $queryString .= " AND "  . $this->wpdb->posts . ".post_status = 'publish'";
        
        // ORDER BY
        $queryString .= " ORDER BY date_begin ASC";
        
        // LIMIT
        if ($limit > 0)
            $queryString .= " LIMIT 0," . $limit;
        
        // get the dates
        $dates = $this->wpdb->get_results($queryString, ARRAY_A);

        // set the classes or the array
        
        $retval = array();
        if (is_array($dates)) {
            for ($i=0;$i<count($dates);$i++) {
                // create models
                $retval[$i] = new eventsbymike_model_date($this->eventsbymike);
                $retval[$i]->setFromArray($dates[$i]);

                if ($return_type) {
                    $retval[$i] = $retval[$i]->getArray();
                }
            }
        } else {
            return FALSE;
        } 


        // array-set of classes/arrays
        return $retval;
    }
    
    /**
     * get Locations Column from DB
     * @param string $searchTerm
     * @return array Result 
     */
    public function getLocations( $searchTerm = '' ) {
        
        //$searchTerm = mysql_real_escape_string($searchTerm);
        
       // $wpdb->get_col('query',column_offset);
        $locations = $this->wpdb->get_results(
                $this->wpdb->prepare("
                                SELECT DISTINCT      location_name
                                FROM        {$this->table_name}
                                WHERE       location_name LIKE '%%%s%%'
                                ORDER BY location_name ASC
                        ", $searchTerm), ARRAY_A);
        
        $retval = array();
        foreach ($locations as $location) {
            $retval[] = $location['location_name'];
        }
                                
        return $retval;
    }


    /**
     * calc Duration
     * @param string $startdate
     * @param string $starttime
     * @param string $enddate
     * @return string Duration
     */
    public function calcDuration ($startdate, $starttime, $enddate) {
        
        // Check Strings
        if (!wpbymike_filter_mysql::checkdate($startdate))
            return FALSE;
        
        if (!wpbymike_filter_mysql::checktime($starttime))
            return FALSE;
        
        // explode strings
        $startdate = explode('-', $startdate);
        $starttime = explode(':', $starttime);

        // create START DateTime Objects
        $startDT = new DateTime();
        $startDT->setDate($startdate[0], $startdate[1], $startdate[2]);
        $startDT->setTime($starttime[0], $starttime[1]);
        
        // create END DateTime Object
        $endDT = new DateTime($enddate);
        
        // calc duration
        $duration = $startDT->diff($endDT);
        
        // RETURN
        return $duration->format('%r%H:%I');
        
    }
    
    /**
     * calc Enddate
     * @param string $startdate
     * @param string $starttime
     * @param string $duration
     * @return string Enddate 
     */
    public function calcEndDate ($startdate, $starttime, $duration) {
        
         // Check Strings
        if (!wpbymike_filter_mysql::checkdate($startdate))
            return FALSE;
        
        if (!wpbymike_filter_mysql::checktime($starttime))
            return FALSE;
        
        if (!wpbymike_filter_mysql::checktime($duration))
            return FALSE;
        
        // explode strings
        $startdate = explode('-', $startdate);
        $starttime = explode(':', $starttime);
        $duration = explode(':', $duration);
       
        // create START DateTime Object
        $startDT = new DateTime();
        $startDT->setDate($startdate[0], $startdate[1], $startdate[2]);
        $startDT->setTime($starttime[0], $starttime[1], 0);
        
        // calc new date
        $startDT->add(DateInterval::createFromDateString($duration[0] . ' Hours + ' . $duration[1] . ' Minutes'));

        // 
        return $startDT->format('Y-m-d H:i:s');
        
    }
    

    /**
     * Check All Values
     * @return boolean 
     */
    public function checkValues() {
        
        if (!wpbymike_filter_mysql::checkdate($this->date_begin))
            return FALSE;
        
        if (!is_integer($this->post_id))
            return FALSE;
        
        if (!wpbymike_filter_mysql::checktime($this->date_time_begin))
            return FALSE;
        
        if (!wpbymike_filter_mysql::checktime($this->date_duration))
            return FALSE;
        
        return TRUE;
            
    }

    /**
     * save to the database
     * @return boolean returns true, if saved.
     */
    public function save() {
        
        if (!$this->checkValues())
            return FALSE;
        
        // Prepare Array with Data
        $saveData = $this->getArray();
        
        // Prepare Duration -> Enddate
        $saveData['date_end'] = $this->calcEndDate($this->date_begin, $this->date_time_begin, $this->date_duration);
        
        // Unset unused Fields
        unset($saveData['date_duration']);
        unset($saveData['date_id']);
            
        
        if (empty($this->date_id)) {
            // Try to insert
            $insert = $this->wpdb->insert(
                    $this->table_name, $saveData, array(
                        '%d', '%s', '%s', '%s', '%s'
            ));
            
            // Check Insert
            if ($insert) {
                $this->setDateId($this->wpdb->insert_id);
                return TRUE; // made insert
            } 
            else return FALSE; // error on insert
            
        }
        
        // Row exists?
        $dbRow = $this->wpdb->get_var(
                $this->wpdb->prepare(
                        "SELECT  {$this->table_name}.date_id
                        FROM    {$this->table_name}
                        WHERE   date_id = %d", $this->getDateId()
                )
        );
                        
        if ($dbRow) {
            $this->wpdb->update( $this->table_name, $saveData, array('date_id' => $this->date_id) );
            return TRUE;
        } 
        
        return FALSE;
        
    }

    /**
     * delete row from database
     * @param int $id date_id, optional
     * @return bool true if deleted
     */
    public function delete($id = NULL) {

        if ($id !== NULL) {
            $this->getById($id);
        }

        if ($this->date_id !== NULL) {

            // Delete the Row use $this->date_id, return the result
            return $this->wpdb->query(
                    $this->wpdb->prepare(
                            "DELETE FROM {$this->table_name}
                                WHERE date_id = %d
                                ", $this->date_id
                    )
            );
                            
        }
        
        return FALSE;
        
    }

    // Useful Functions

    /**
     * Set Values from an Array
     * @param array $newvalues 
     */
    public function setFromArray($newvalues = array()) {

        // Merge with Presets
        $newvalues = array_merge($this->presets, $newvalues);

        // Check & set Duration
        if (isset($newvalues['date_end'])) {
            $newvalues['date_duration'] = $this->calcDuration(
                    $newvalues['date_begin'],
                    $newvalues['date_time_begin'],
                    $newvalues['date_end']
            );
            unset($newvalues['date_end']);
        }
            
        // Set Values
        $this->setDateId($newvalues['date_id']);
        $this->setPostId($newvalues['post_id']);
        $this->setDateBegin($newvalues['date_begin']);
        $this->setDateTimeBegin($newvalues['date_time_begin']);
        $this->setDateDuration($newvalues['date_duration']);
        $this->setLocationName($newvalues['location_name']);
        
        // Return this Object for further usage
        return $this;
    }

    /**
     * Returns all Values in an Array
     * @return array
     */
    public function getArray() {

        // First get Presets
        $retval = $this->presets;

        // Get the Values
        $retval['date_id'] = $this->getDateId();
        $retval['post_id'] = $this->getPostId();
        $retval['date_begin'] = $this->getDateBegin();
        $retval['date_time_begin'] = $this->getDateTimeBegin();
        $retval['date_duration'] = $this->getDateDuration();
        $retval['location_name'] = $this->getLocationName();

        // Return the array;
        return $retval;
    }

    // GETTERS & SETTERS

    /**
     * Get date_id
     * @return int date_id 
     */
    public function getDateId() {
        return $this->date_id;
    }

    /**
     * Set date_id
     * @param int $id 
     */
    public function setDateId($id) {
        $this->date_id = (int) $id;
    }

    /**
     * get post_id
     * @return int post_id 
     */
    public function getPostId() {
        return $this->post_id;
    }

    /**
     * set post_id
     * @param int $PostId 
     */
    public function setPostId($PostId) {
        $this->post_id = (int) $PostId;
    }

    /**
     * get date_begin
     * @return string date_begin 
     */
    public function getDateBegin() {
        return $this->date_begin;
    }

    /**
     * set date_begin
     * @param string $date_begin 
     */
    public function setDateBegin($date_begin) {
        $this->date_begin = $date_begin;
    }

    /**
     * get date_begin
     * @return string date_begin 
     */
    public function getDateTimeBegin() {
        return $this->date_time_begin;
    }

    /**
     * set date_begin
     * @param string $date_begin 
     */
    public function setDateTimeBegin($date_time_begin) {
        $this->date_time_begin = substr($date_time_begin, 0, 5);
    }
    
    /**
     * get date_begin
     * @return string date_begin 
     */
    public function getDateDuration() {
        return $this->date_duration;
    }

    /**
     * set date_begin
     * @param string $date_begin 
     */
    public function setDateDuration($date_duration) {
        $this->date_duration = $date_duration;
    }


    /**
     * get $location_name
     * @return string $location_name 
     */
    public function getLocationName() {
        return $this->location_name;
    }

    /**
     * set $location_name
     * @param string $location_name 
     */
    public function setLocationName($location_name) {
        $this->location_name = $location_name;
    }

}

// EoC eventsbymike_model_date