<?php
/*
  Plugin Name: Events byMike
  Plugin URI: http://events.bymike.ch/
  Description: Ein PlugIn zur Anzeige von Auftrittsterminen.
  Author: byMike.ch
  Author URI: http://bymike.ch
  Version: 0.5
 */

/*  Copyright (C)2012 byMike <m at byMike.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// INCLUDES

include dirname(__FILE__) . '/lib/autoloader.php';

// Start Autoloader
$eventsbymike_autoloader = new wpbymike_autoloader( array(
    'basePath' => plugin_dir_path(__FILE__),
    'sources' => array (
        'eventsbymike' => 'app',
        'wpbymike' => 'lib'
    )
) );


/**
 * Eventsbymike PlugIn-Class
 *
 * @author byMike <m at byMike.ch>
 * 
 */


if (!class_exists('eventsbymike')) {

    class eventsbymike extends wpbymike_plugin {

        /**
         * @var int actual & needed db-table version
         */
        public static $dbVersion = 1;
        
        /**
         * @var int actual plugin version
         */
        public static $pluginVersion = 1;
        
        /**
         * @var boolean turns debugging on||off for this plugin
         */
        public static $EBM_DEBUG = FALSE; // @TODO SET THIS TO FALSE BEFORE PUBLISHING.
        
        /**
         * @var array plugin configuration
         */
        protected $options = array(
            'db_table_name' => 'eventsbymike',
            'viewDir' => 'app/view/'
        );
        
        /**
         * @var string database table name
         */
        public $table_name = '';
        
        /**
         *  @var $viewVars array stores variables for view
		 */
        protected $viewVars = array();
        
        /**
         * @var string name of class & plugin
         */
        public $pluginName = 'eventsbymike';
        
        /**
         * @var string complete plugin path
         */
        public $pluginDir = '';
        
        /**
         * @var string url to plugin
         */
        public $pluginDirUrl = '';
        
        /**
         * Constructor 
         */
        function __construct() {

            register_activation_hook(__FILE__, array($this, 'activate'));
            register_deactivation_hook(__FILE__, array($this, 'deactivate'));
            register_uninstall_hook(__FILE__, array('eventsbymike', 'uninstall'));
            
            //$this->pluginName = plugin_basename(__FILE__);
            $this->pluginDir = plugin_dir_path(__FILE__);
            $this->pluginDirUrl = plugin_dir_url(__FILE__);

            // get complete table name
            $this->table_name = $this->getwpdb()->prefix . $this->options['db_table_name'];
            $this->options['db_table_name'] = $this->table_name;
            
        }
        
        /**
         * Initialize the Plugin 
         */
        function init() {
        	
        	// check for updates
        	if (get_option('eventsbymike_plugin_version', 0) != self::$pluginVersion)
        		$this->updatePlugIn();
            
            // hier setzen wir unsere Hooks, z.B.:
            add_action('add_meta_boxes', array($this, 'addMetaboxes'));
            add_action('save_post', array($this, 'myplugin_save_postdata'));
            add_action('delete_post', array($this, 'deletePost'));

            
            // Add ShortCodes
            add_shortcode( 'ebm_dates', array($this->getController('shortcodes'), 'ebm_dates'));
            add_shortcode( 'ebm_vita', array($this->getController('shortcodes'), 'ebm_vita'));
            
            // ADMIN SECTION
            if (is_admin()) {
                
                // Register Admin Ajax Response Functions
                add_action('wp_ajax_eventsbymike_checkdate', array($this->getController('postMetabox'), 'ajaxCheckdate'));
                add_action('wp_ajax_eventsbymike_deletedate', array($this->getController('postMetabox'), 'ajaxDeletedate'));
                add_action('wp_ajax_eventsbymike_savedates', array($this->getController('postMetabox'), 'ajaxSaveDates'));
                add_action('wp_ajax_eventsbymike_autocomplete', array($this->getController('postMetabox'), 'ajaxAutocomplete'));
                
                // Enque Scripts
                wp_enqueue_script('jquery');
                wp_enqueue_script('jquery-ui-core');
                wp_enqueue_script('jquery-ui-datepicker');
                wp_enqueue_script( 'ebm_timepicker', $this->pluginDirUrl . 'js/jquery.timePicker.min.js', array('jquery', 'jquery-ui-core'));
                wp_enqueue_script( 'ebm_jsfuncs', $this->pluginDirUrl . 'js/eventsbymike_jsfunctions.js', array('jquery', 'jquery-ui-core'));
                wp_enqueue_script( 'ebm_bsn', $this->pluginDirUrl . 'js/bsn.AutoSuggest_2.1.3_comp.js', array('jquery', 'jquery-ui-core'));
            }
        }

        /**
         * Plugin Activation
         */
        function activate() {
            
            // Install DB
            $this->dbInstall();
            
            // @TODO Check Dates Post-Id's, if there are some deleted Posts. It's a Clean Up Procedure.

        }
        
        /**
         * PlugIn Update Procedure
         */
        function updatePlugIn() {
        	
        	// Check DB
        	$this->dbInstall();
        	
        	// set to actual version
        	add_option('eventsbymike_plugin_version', self::$pluginVersion);
        }
        
        /**
         * PlugIn Uninstall Procedure
         */
        public static function uninstall() {
        	//if ( ! defined('WP_UNINSTALL_PLUGIN') ) <--- will not work yet
        	//return;
        	
        	$ebmObj = new eventsbymike();
        	
        	$ebmObj->dbUnInstall();
        	
        	delete_option('eventsbymike_plugin_version');
        	
        	
        }
        
        /**
         * Plugin DB-Table Uninstall
         */
        public function dbUnInstall() {
        	global $wpdb;
        	
        	echo "Uninstall TABLE...\n";
        	$sql = "DROP TABLE IF EXISTS `{$this->table_name}`";
        	if (!empty($sql))
        		$wpdb->query($sql);
        	delete_option('eventsbymike_db_version');
        	echo "Uninstalled...\n";
        }

        
        /**
         * Plugin deactivation 
         */
        function deactivate() {
        	
        	// there's nothing to do yet.
 
        }
 
        /**
         * Database Table Installer 
         */
        function dbInstall() {

            // get Upgrade Functions
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            
            // Prepare
            $sql = NULL;
            
            // Check DB Version
            switch (get_option('eventsbymike_db_version', 0)) {
                case 1:
                    break;

                default:
                    $sql = "CREATE TABLE IF NOT EXISTS `{$this->table_name}` (
                            `date_id` int(11) NOT NULL AUTO_INCREMENT,
                            `post_id` int(11) NOT NULL,
                            `date_begin` date NOT NULL,
                            `date_time_begin` time NOT NULL,
                            `date_end` datetime DEFAULT NULL,
                            `location_name` text,
                            PRIMARY KEY (`date_id`)
                            ) ENGINE=InnoDB  DEFAULT CHARSET=utf8 ;";
                    break;
            }
            if (!empty($sql))
                dbDelta($sql);

            add_option('eventsbymike_db_version', self::$dbVersion);
        
            
        }
        
        public function deletePost ($postId) {
            $dateModel = new eventsbymike_model_date($this);
            $dates = $dateModel->getAllByPostId($postId);
            
            // delete them
            foreach ($dates as $date) {
                $date->delete();
            }
        }
        
        
        
        /**
         * Adds the meta box container
         */
        public function addMetaboxes() {
            add_meta_box(
                    'ebm_postMetabox'
                    , 'Dates (eventsByMike)'
                    , array($this->getController('postMetabox'), 'getMetabox')
                    , 'post'
                    , 'advanced'
                    , 'high'
            );
        }
        
  
        /**
         *  When the post is saved, saves our custom data
         */
        function myplugin_save_postdata($post_id) {

            // Check if this is a post
            if (isset($_POST['post_type']) && ('post' <> $_POST['post_type'])) {
                return;
            }
            
            // verify if this is an auto save routine. 
            // If it is our form has not been submitted, so we dont want to do anything
            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
                return FALSE;

            // verify this came from the our screen and with proper authorization,
            // because save_post can be triggered at other times
            if (isset($_POST['eventsbymike_nonce'])) {
                if (!wp_verify_nonce($_POST['eventsbymike_nonce'], 'eventsbymike-metabox'))
                    return FALSE;
            }

            // Check permissions
            if (!current_user_can('edit_post', $post_id)) {
                return FALSE;
            }

            // OK, we're authenticated: we need to find and save the data

            $newdates = $_POST['datebyMike'];



            if (count($newdates) > 0) {
                foreach ($newdates as $newdate) {
                    if($newdate['date_begin'] <> '0000-00-00') {
                        // create the model
                        $datemodel = new eventsbymike_model_date($this, $newdate);
                        $datemodel->setPostId($post_id);
                        $datemodel->save();
                        unset($datemodel);
                    }
                }
            }
        }

    }

} // EoC eventsbymike
// REGISTER
if (class_exists('eventsbymike')) {
    $eventsByMike = new eventsbymike();
}

if (isset($eventsByMike)) {
    add_action('init', array($eventsByMike, 'init'));
}

// UNINSTALL



