<?php
/*  Copyright (C)2012 byMike <m at byMike.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Description of autoloader
 *
 * @author byMike <m at byMike.ch>
 * 
 */
class wpbymike_autoloader {
    
    protected $preDefSources = array(
        'eventsbymike' => 'app',
        'wpbymike' => 'lib'
    );
    
    protected $basePath = NULL;
    
    public function __construct($config = array()) {
        
        // Set Config
        if (isset($config['basePath']))
            $this->basePath = $config['basePath'];
        else
            $this->basePath = dirname (__FILE__) . '../';
        
        // Set Predefined Sources
        if (isset($config['sources'])) 
            $this->setSources($config['sources']);
        
        // Register Autoloader
        spl_autoload_register(array($this, 'load'));
        
        
    }
    
    /**
     * Set Sources Options
     * @param array $array ( 'namespace' => 'path' ) 
     */
    public function setSources ($array = array()) {
        if (is_array($array))
            $this->preDefSources = $array;
    }


    public function load ($classname) {
        if ( strpos ($classname, '.') !== false
                || strpos ($classname, '/') !== false
                || strpos ($classname, '\\') !== false
                || strpos ($classname, ':') !== false )
        {
            return;
        }
        
        $class = explode('_',$classname);
        
        $includeString = $this->basePath;
        
        $firstVal = TRUE;
        
        foreach ($class as $pathPart) {
            
            if (!$firstVal) $includeString .= '/';
            else $firstVal = FALSE;
            
            if (isset($this->preDefSources[$pathPart])) {
                $includeString .= $this->preDefSources[$pathPart];
            } else {
                $includeString .= $pathPart;
            }
        }
        
        $includeString .= '.php';
        
        if (file_exists($includeString)) {
            include_once $includeString;
        }
    }
}

// EoC autoloader