<?php
/*  Copyright (C)2012 byMike <m at byMike.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Description of wpbymike_plugin
 *
 * @author byMike <m at byMike.ch>
 * 
 */
abstract class wpbymike_plugin
{
    protected $options = array();
    
    protected $viewObj = NULL;
    
    protected $controllerInstances = array();


    /**
     * Get the View Object
     * 
     * @param array $config
     * @return \wpbymike_view object 
     */
    public function view($config = array()) {
        if (is_null($this->viewObj)) {
            $this->viewObj = new wpbymike_view ($this->options);
            $this->viewObj->setOptions($config);

            // Register Std ViewVars
            // Plugin Information
            $this->viewObj->registerVars( array(
                'pluginDir' => $this->pluginDir,
                'pluginDirUrl' => $this->pluginDirUrl,
                'pluginName' => $this->pluginName
            ));

            // Set Controller Instance
            $this->viewObj->setController($this);
        }

        return $this->viewObj;

    }
    
    protected function getController($controllerName) {
        $className = $this->pluginName . '_controller_' . $controllerName;
        if (!isset($this->controllerInstances[$controllerName])) {
            $this->controllerInstances[$controllerName] = new $className($this);
        }
        
        return $this->controllerInstances[$controllerName];
    }
    
    /**
     * Returns the wpdb-object. 
     */
    public function getwpdb() {
        global $wpdb;
        return $wpdb;
    }
        
    /**
     * return option val
     * @param type $option
     * @return mixed option or FALSE, if option is not set
     */
    public function getOption ( $option ) {
        if (isset($this->options[$option])) {
            return $this->options[$option];
        } else {
            return FALSE;
        }
    }
    
    /**
     * Gets all options as array
     * @param array $optionKeys Optional
     * @return array $options 
     */
    public function getOptions ( $optionKeys = NULL ) {
        
        if (is_array($optionKeys))
            return array_combine($optionKeys, $this->options);
        
        return $this->options;
    }
    
    /**
     * Set an Option in the Option Array.
     * @param string/array $optionKey / ArrayMap with key/val Option-Pairs
     * @param string $optionVal Optional
     */
    public function setOption ( $optionKey, $optionVal = NULL ) {
        if (is_array($optionKey)) {
            $this->options = array_merge($optionKey, $this->options);
        } else {
            $this->options[$optionKey] = $optionVal;
        }
    }
    
    /**
     * Set Options from arrayMap
     * @param array $optionMap 
     */
    public function setOptions($optionMap) {
        $this->setOption($optionMap);
    }
    
        

}

// EoC wpbymike_plugin