<?php

/*  Copyright (C)2012 byMike <m at byMike.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Predefined View Object
 *
 * @author byMike <m at byMike.ch>
 * 
 */
class wpbymike_view extends wpbymike_baseClass {
    
    protected $viewVars = array();
    
    protected $pathToViewFiles = NULL;
    
    protected $viewFiles = array();
    
    protected $controllerInstance = NULL;

        
    /**
     * Construct View
     * @param arraymap $config 
     */
    public function __construct($config = array()) {
        
        if (is_array($config)) {
            $this->setOptions($config);
        }
    }
    
    /**
     * returns ControllerInstance
     * @return \eventsbymike
     */
    public function getController () {
        return $this->controllerInstance;
    }
    
    /**
     * set ControllerInstance
     * @param type $controllerInstance 
     */
    public function setController ($controllerInstance) {
        $this->controllerInstance = $controllerInstance;
    }

    /**
    * Register Vars for View
    * @param string/array $varName Name of the var or an array-set of values
    * @param var $varValue
    */
    public function registerVar ($varName, $varValue = '') {
        if (is_array($varName)) {
            foreach ($varName as $key => $value) {
                $this->viewVars[$key] = $value;
            }
        } elseif (!empty($varName)) {
            $this->viewVars[$varName] = $varValue;
        } 
    }
    
    /**
     * passes the arrayMap to registerVar
     * @param type $arrayMap 
     */
    public function registerVars($arrayMap = array()) {
        if (is_array($arrayMap))
            $this->registerVar ($arrayMap);
        
    }
    
    public function getViewVar($varName) {
        if (isset($this->viewVars[$varName]))
            return $this->viewVars[$varName];
        else return NULL;
    }


    /**
     * register Viewfile
     * @param string/arrayMap $viewFile 
     */
    public function registerViewFile($viewFile) {
        if (is_array($viewFile)) {
            foreach ($viewFile as $file) {
                $this->viewFiles[] = $file;
            }
        } else {
            $this->viewFiles[] = $viewFile;
        }
            
    }


    /**
     * Echoes the viewFile
     * @param string/arrayMap $viewFile path to view File(s)
     * @param boolean [optional] Render all registred files, or just $viewFile(s) 
     * @param boolean [optional] Return the output instead Echo
     * @return string
     */
    public function render ($viewFile = NULL, $renderAll = TRUE, $return = FALSE) {
        
        // TODO $viewFile should be Optional!
        // TODO Arraymap $viewFiles
        
        // Check Args
        if (!is_array($viewFile)) {
            $viewFile = array( $viewFile );
        }
        
        // Prepare the Registred Vars
        foreach ($this->viewVars as $key => $val) {
            $$key = $val;
        }
        
        // Files to render
        $files = array();
        if ($renderAll) {
            $files = $this->viewFiles;
        }
        $files = $files + $viewFile;
        
        
        // Start Buffer
        ob_start();
        
        // include the files
        foreach ($files as $file) {
            if (file_exists($this->getViewVar('pluginDir') . $this->getOption('viewDir') . $file))
               include $this->getViewVar('pluginDir') . $this->getOption('viewDir') . $file;
        }
        
        if ($return)
            return ob_get_clean();
        else echo ob_get_clean ();
    }
}

// EoC view